? diff-result
Index: src/imap/cmd-expunge.c
===================================================================
RCS file: /home/cvs/dovecot/src/imap/cmd-expunge.c,v
retrieving revision 1.5
diff -u -r1.5 cmd-expunge.c
--- src/imap/cmd-expunge.c	27 Jul 2003 04:48:32 -0000	1.5
+++ src/imap/cmd-expunge.c	20 Nov 2003 16:35:50 -0000
@@ -9,10 +9,12 @@
 	if (!client_verify_open_mailbox(client))
 		return TRUE;
 
-	if (imap_expunge(client->mailbox, TRUE))
-		client_send_tagline(client, "OK Expunge completed.");
+	if (imap_expunge(client->mailbox, TRUE)) {
+                client_sync_message_counts(client);
+                client_send_tagline(client, "OK Expunge completed.");
+        }
 	else
 		client_send_storage_error(client, client->mailbox->storage);
-
+        
 	return TRUE;
 }
Index: src/imap/commands-util.c
===================================================================
RCS file: /home/cvs/dovecot/src/imap/commands-util.c,v
retrieving revision 1.29
diff -u -r1.29 commands-util.c
--- src/imap/commands-util.c	24 Aug 2003 12:45:32 -0000	1.29
+++ src/imap/commands-util.c	20 Nov 2003 16:35:51 -0000
@@ -144,6 +144,18 @@
 	}
 }
 
+void client_sync_message_counts(struct client *client)
+{
+	if (client->mailbox == NULL)
+		return;
+
+	if (!client->mailbox->sync(client->mailbox,
+                MAILBOX_SYNC_FORCE_COUNT)) {
+		client_send_untagged_storage_error(client,
+                    client->mailbox->storage);
+	}
+}
+
 void client_send_storage_error(struct client *client,
 			       struct mail_storage *storage)
 {
Index: src/imap/commands-util.h
===================================================================
RCS file: /home/cvs/dovecot/src/imap/commands-util.h,v
retrieving revision 1.13
diff -u -r1.13 commands-util.h
--- src/imap/commands-util.h	27 Jul 2003 04:48:32 -0000	1.13
+++ src/imap/commands-util.h	20 Nov 2003 16:35:52 -0000
@@ -31,6 +31,9 @@
 /* Synchronize all but expunges with client. */
 void client_sync_without_expunges(struct client *client);
 
+/* Synchronize messages counts with client. */
+void client_sync_message_counts(struct client *client);
+
 /* Send last mail storage error message to client. */
 void client_send_storage_error(struct client *client,
 			       struct mail_storage *storage);
Index: src/lib-storage/mail-storage.h
===================================================================
RCS file: /home/cvs/dovecot/src/lib-storage/mail-storage.h,v
retrieving revision 1.62
diff -u -r1.62 mail-storage.h
--- src/lib-storage/mail-storage.h	26 Oct 2003 20:13:15 -0000	1.62
+++ src/lib-storage/mail-storage.h	20 Nov 2003 16:35:55 -0000
@@ -99,7 +99,8 @@
 	MAILBOX_SYNC_NONE		= 0x00,
 	MAILBOX_SYNC_FULL		= 0x01,
 	MAILBOX_SYNC_FAST		= 0x02,
-	MAILBOX_SYNC_FLAG_NO_EXPUNGES	= 0x04
+	MAILBOX_SYNC_FLAG_NO_EXPUNGES	= 0x04,
+	MAILBOX_SYNC_FORCE_COUNT        = 0x08
 };
 
 enum client_workarounds {
Index: src/lib-storage/index/index-sync.c
===================================================================
RCS file: /home/cvs/dovecot/src/lib-storage/index/index-sync.c,v
retrieving revision 1.27
diff -u -r1.27 index-sync.c
--- src/lib-storage/index/index-sync.c	24 Aug 2003 12:45:33 -0000	1.27
+++ src/lib-storage/index/index-sync.c	20 Nov 2003 16:35:56 -0000
@@ -10,7 +10,7 @@
 /* How often to do full sync when fast sync flag is set. */
 #define MAILBOX_FULL_SYNC_INTERVAL 5
 
-static void index_storage_sync_size(struct index_mailbox *ibox)
+static void index_storage_sync_size(struct index_mailbox *ibox, int force)
 {
 	struct mail_storage *storage = ibox->box.storage;
 	unsigned int messages, recent;
@@ -21,9 +21,11 @@
 	messages = ibox->index->get_header(ibox->index)->messages_count;
 	messages += mail_modifylog_get_expunge_count(ibox->index->modifylog);
 
-	if (messages != ibox->synced_messages_count) {
-		i_assert(messages > ibox->synced_messages_count);
-
+	if ((messages != ibox->synced_messages_count) || force) {
+                if (!force) {
+                        i_assert(messages > ibox->synced_messages_count);
+                }
+                
 		/* new messages in mailbox */
 		recent = index_storage_get_recent_count(ibox->index);
 		storage->callbacks->new_messages(&ibox->box, messages, recent,
@@ -82,7 +84,7 @@
 		return TRUE; /* no changes - must be no new mail either */
 
 	if (sync_size)
-		index_storage_sync_size(ibox);
+		index_storage_sync_size(ibox, 0);
 
 	/* notify changes in custom flags */
 	if (mail_custom_flags_has_changes(index->custom_flags) &&
@@ -240,11 +242,15 @@
 		ret = index_storage_sync_modifylog(ibox, FALSE);
 	else
 		ret = TRUE;
-
-	index_storage_sync_size(ibox);
-
+        
+        if (flags & MAILBOX_SYNC_FORCE_COUNT)
+                index_storage_sync_size(ibox, 1);
+        else
+                index_storage_sync_size(ibox, 0);
+        
 	if (!index_storage_lock(ibox, MAIL_LOCK_UNLOCK))
 		return FALSE;
 
 	return ret;
 }
+
